@@ -1,31 +1,41 @@
+"""Fake mixer for use in tests.
+
+**Dependencies:**
+
+- None
+
+**Settings:**
+
+- Set :attr:`mopidy.settings.MIXER` to ``fakemixer`` to use this mixer.
+"""
+
+from __future__ import unicode_literals
+
 import pygst
 pygst.require('0.10')
 import gobject
 import gst
 
-from mopidy.audio.mixers import create_track
+from . import utils
 
 
 class FakeMixer(gst.Element, gst.ImplementsInterface, gst.interfaces.Mixer):
-    __gstdetails__ = ('FakeMixer',
-                      'Mixer',
-                      'Fake mixer for use in tests.',
-                      'Thomas Adamcik')
+    __gstdetails__ = (
+        'FakeMixer',
+        'Mixer',
+        'Fake mixer for use in tests.',
+        'Mopidy')
 
     track_label = gobject.property(type=str, default='Master')
     track_initial_volume = gobject.property(type=int, default=0)
     track_min_volume = gobject.property(type=int, default=0)
     track_max_volume = gobject.property(type=int, default=100)
     track_num_channels = gobject.property(type=int, default=2)
-    track_flags = gobject.property(type=int,
-        default=(gst.interfaces.MIXER_TRACK_MASTER |
-                 gst.interfaces.MIXER_TRACK_OUTPUT))
-
-    def __init__(self):
-        gst.Element.__init__(self)
+    track_flags = gobject.property(type=int, default=(
+        gst.interfaces.MIXER_TRACK_MASTER | gst.interfaces.MIXER_TRACK_OUTPUT))
 
     def list_tracks(self):
-        track = create_track(
+        track = utils.create_track(
             self.track_label,
             self.track_initial_volume,
             self.track_min_volume,
@@ -42,7 +52,3 @@ class FakeMixer(gst.Element, gst.ImplementsInterface, gst.interfaces.Mixer):
 
     def set_record(self, track, record):
         pass
-
-
-gobject.type_register(FakeMixer)
-gst.element_register(FakeMixer, 'fakemixer', gst.RANK_MARGINAL)
