@@ -1,7 +1,10 @@
+from __future__ import unicode_literals
+
 from mopidy import settings
 from mopidy.frontends.mpd.protocol import handle_request
-from mopidy.frontends.mpd.exceptions import (MpdPasswordError,
-    MpdPermissionError)
+from mopidy.frontends.mpd.exceptions import (
+    MpdPasswordError, MpdPermissionError)
+
 
 @handle_request(r'^close$', auth_required=False)
 def close(context):
@@ -14,6 +17,7 @@ def close(context):
     """
     context.session.close()
 
+
 @handle_request(r'^kill$')
 def kill(context):
     """
@@ -23,7 +27,8 @@ def kill(context):
 
         Kills MPD.
     """
-    raise MpdPermissionError(command=u'kill')
+    raise MpdPermissionError(command='kill')
+
 
 @handle_request(r'^password "(?P<password>[^"]+)"$', auth_required=False)
 def password_(context, password):
@@ -38,7 +43,8 @@ def password_(context, password):
     if password == settings.MPD_SERVER_PASSWORD:
         context.dispatcher.authenticated = True
     else:
-        raise MpdPasswordError(u'incorrect password', command=u'password')
+        raise MpdPasswordError('incorrect password', command='password')
+
 
 @handle_request(r'^ping$', auth_required=False)
 def ping(context):
