@@ -1,5 +1,7 @@
+from __future__ import unicode_literals
+
 from mopidy.frontends.mpd.protocol import handle_request, mpd_commands
-from mopidy.frontends.mpd.exceptions import MpdNotImplemented
+
 
 @handle_request(r'^commands$', auth_required=False)
 def commands(context):
@@ -13,16 +15,20 @@ def commands(context):
     if context.dispatcher.authenticated:
         command_names = set([command.name for command in mpd_commands])
     else:
-        command_names = set([command.name for command in mpd_commands
+        command_names = set([
+            command.name for command in mpd_commands
             if not command.auth_required])
 
     # No one is permited to use kill, rest of commands are not listed by MPD,
     # so we shouldn't either.
-    command_names = command_names - set(['kill', 'command_list_begin',
-        'command_list_ok_begin', 'command_list_ok_begin', 'command_list_end',
-        'idle', 'noidle', 'sticker'])
+    command_names = command_names - set([
+        'kill', 'command_list_begin', 'command_list_ok_begin',
+        'command_list_ok_begin', 'command_list_end', 'idle', 'noidle',
+        'sticker'])
+
+    return [
+        ('command', command_name) for command_name in sorted(command_names)]
 
-    return [('command', command_name) for command_name in sorted(command_names)]
 
 @handle_request(r'^decoders$')
 def decoders(context):
@@ -40,8 +46,16 @@ def decoders(context):
             mime_type: audio/mpeg
             plugin: mpcdec
             suffix: mpc
+
+    *Clarifications:*
+
+    - ncmpcpp asks for decoders the first time you open the browse view. By
+      returning nothing and OK instead of an not implemented error, we avoid
+      "Not implemented" showing up in the ncmpcpp interface, and we get the
+      list of playlists without having to enter the browse interface twice.
     """
-    raise MpdNotImplemented # TODO
+    return  # TODO
+
 
 @handle_request(r'^notcommands$', auth_required=False)
 def notcommands(context):
@@ -55,13 +69,15 @@ def notcommands(context):
     if context.dispatcher.authenticated:
         command_names = []
     else:
-        command_names = [command.name for command in mpd_commands
-            if command.auth_required]
+        command_names = [
+            command.name for command in mpd_commands if command.auth_required]
 
     # No permission to use
     command_names.append('kill')
 
-    return [('command', command_name) for command_name in sorted(command_names)]
+    return [
+        ('command', command_name) for command_name in sorted(command_names)]
+
 
 @handle_request(r'^tagtypes$')
 def tagtypes(context):
@@ -72,7 +88,8 @@ def tagtypes(context):
 
         Shows a list of available song metadata.
     """
-    pass # TODO
+    pass  # TODO
+
 
 @handle_request(r'^urlhandlers$')
 def urlhandlers(context):
@@ -83,5 +100,6 @@ def urlhandlers(context):
 
         Gets a list of available URL handlers.
     """
-    return [(u'handler', uri_scheme)
-        for uri_scheme in context.backend.uri_schemes.get()]
+    return [
+        ('handler', uri_scheme)
+        for uri_scheme in context.core.uri_schemes.get()]
