@@ -1,5 +1,6 @@
-# Absolute import needed to import ~/.mopidy/settings.py and not ourselves
-from __future__ import absolute_import
+# Absolute import needed to import ~/.config/mopidy/settings.py and not
+# ourselves
+from __future__ import absolute_import, unicode_literals
 
 import copy
 import getpass
@@ -8,9 +9,8 @@ import os
 import pprint
 import sys
 
-from mopidy import SettingsError, SETTINGS_PATH, SETTINGS_FILE
-from mopidy.utils import log
-from mopidy.utils import path
+from mopidy import exceptions
+from mopidy.utils import formatting, path
 
 logger = logging.getLogger('mopidy.utils.settings')
 
@@ -23,16 +23,17 @@ class SettingsProxy(object):
         self.runtime = {}
 
     def _get_local_settings(self):
-        if not os.path.isfile(SETTINGS_FILE):
+        if not os.path.isfile(path.SETTINGS_FILE):
             return {}
-        sys.path.insert(0, SETTINGS_PATH)
+        sys.path.insert(0, path.SETTINGS_PATH)
         # pylint: disable = F0401
         import settings as local_settings_module
         # pylint: enable = F0401
         return self._get_settings_dict_from_module(local_settings_module)
 
     def _get_settings_dict_from_module(self, module):
-        settings = filter(lambda (key, value): self._is_setting(key),
+        settings = filter(
+            lambda (key, value): self._is_setting(key),
             module.__dict__.iteritems())
         return dict(settings)
 
@@ -50,13 +51,13 @@ class SettingsProxy(object):
         if not self._is_setting(attr):
             return
 
-        current = self.current # bind locally to avoid copying+updates
+        current = self.current  # bind locally to avoid copying+updates
         if attr not in current:
-            raise SettingsError(u'Setting "%s" is not set.' % attr)
+            raise exceptions.SettingsError('Setting "%s" is not set.' % attr)
 
         value = current[attr]
         if isinstance(value, basestring) and len(value) == 0:
-            raise SettingsError(u'Setting "%s" is empty.' % attr)
+            raise exceptions.SettingsError('Setting "%s" is empty.' % attr)
         if not value:
             return value
         if attr.endswith('_PATH') or attr.endswith('_FILE'):
@@ -73,22 +74,25 @@ class SettingsProxy(object):
         if interactive:
             self._read_missing_settings_from_stdin(self.current, self.runtime)
         if self.get_errors():
-            logger.error(u'Settings validation errors: %s',
-                log.indent(self.get_errors_as_string()))
-            raise SettingsError(u'Settings validation failed.')
+            logger.error(
+                'Settings validation errors: %s',
+                formatting.indent(self.get_errors_as_string()))
+            raise exceptions.SettingsError('Settings validation failed.')
 
     def _read_missing_settings_from_stdin(self, current, runtime):
         for setting, value in sorted(current.iteritems()):
             if isinstance(value, basestring) and len(value) == 0:
-                runtime[setting] = self._read_from_stdin(setting + u': ')
+                runtime[setting] = self._read_from_stdin(setting + ': ')
 
     def _read_from_stdin(self, prompt):
-        if u'_PASSWORD' in prompt:
-            return (getpass.getpass(prompt)
+        if '_PASSWORD' in prompt:
+            return (
+                getpass.getpass(prompt)
                 .decode(sys.stdin.encoding, 'ignore'))
         else:
             sys.stdout.write(prompt)
-            return (sys.stdin.readline().strip()
+            return (
+                sys.stdin.readline().strip()
                 .decode(sys.stdin.encoding, 'ignore'))
 
     def get_errors(self):
@@ -97,7 +101,7 @@ class SettingsProxy(object):
     def get_errors_as_string(self):
         lines = []
         for (setting, error) in self.get_errors().iteritems():
-            lines.append(u'%s: %s' % (setting, error))
+            lines.append('%s: %s' % (setting, error))
         return '\n'.join(lines)
 
 
@@ -117,7 +121,6 @@ def validate_settings(defaults, settings):
     errors = {}
 
     changed = {
-        'CUSTOM_OUTPUT': 'OUTPUT',
         'DUMP_LOG_FILENAME': 'DEBUG_LOG_FILENAME',
         'DUMP_LOG_FORMAT': 'DEBUG_LOG_FORMAT',
         'FRONTEND': 'FRONTENDS',
@@ -139,43 +142,45 @@ def validate_settings(defaults, settings):
         'SPOTIFY_LIB_CACHE': 'SPOTIFY_CACHE_PATH',
     }
 
+    list_of_one_or_more = [
+        'BACKENDS',
+        'FRONTENDS',
+    ]
+
     for setting, value in settings.iteritems():
         if setting in changed:
             if changed[setting] is None:
-                errors[setting] = u'Deprecated setting. It may be removed.'
+                errors[setting] = 'Deprecated setting. It may be removed.'
             else:
-                errors[setting] = u'Deprecated setting. Use %s.' % (
+                errors[setting] = 'Deprecated setting. Use %s.' % (
                     changed[setting],)
 
-        elif setting == 'BACKENDS':
-            if 'mopidy.backends.despotify.DespotifyBackend' in value:
-                errors[setting] = (
-                    u'Deprecated setting value. '
-                    u'"mopidy.backends.despotify.DespotifyBackend" is no '
-                    u'longer available.')
-
         elif setting == 'OUTPUTS':
             errors[setting] = (
-                u'Deprecated setting, please change to OUTPUT. OUTPUT expects '
-                u'a GStreamer bin description string for your desired output.')
+                'Deprecated setting, please change to OUTPUT. OUTPUT expects '
+                'a GStreamer bin description string for your desired output.')
 
         elif setting == 'SPOTIFY_BITRATE':
             if value not in (96, 160, 320):
                 errors[setting] = (
-                    u'Unavailable Spotify bitrate. Available bitrates are 96, '
-                    u'160, and 320.')
+                    'Unavailable Spotify bitrate. Available bitrates are 96, '
+                    '160, and 320.')
 
         elif setting.startswith('SHOUTCAST_OUTPUT_'):
             errors[setting] = (
-                u'Deprecated setting, please set the value via the GStreamer '
-                u'bin in OUTPUT.')
+                'Deprecated setting, please set the value via the GStreamer '
+                'bin in OUTPUT.')
 
-        elif setting not in defaults:
-            errors[setting] = u'Unknown setting.'
+        elif setting in list_of_one_or_more:
+            if not value:
+                errors[setting] = 'Must contain at least one value.'
+
+        elif setting not in defaults and not setting.startswith('CUSTOM_'):
+            errors[setting] = 'Unknown setting.'
             suggestion = did_you_mean(setting, defaults)
 
             if suggestion:
-                errors[setting] += u' Did you mean %s?' % suggestion
+                errors[setting] += ' Did you mean %s?' % suggestion
 
     return errors
 
@@ -198,19 +203,20 @@ def format_settings_list(settings):
     for (key, value) in sorted(settings.current.iteritems()):
         default_value = settings.default.get(key)
         masked_value = mask_value_if_secret(key, value)
-        lines.append(u'%s: %s' % (
-            key, log.indent(pprint.pformat(masked_value), places=2)))
+        lines.append('%s: %s' % (
+            key, formatting.indent(pprint.pformat(masked_value), places=2)))
         if value != default_value and default_value is not None:
-            lines.append(u'  Default: %s' %
-                log.indent(pprint.pformat(default_value), places=4))
+            lines.append(
+                '  Default: %s' %
+                formatting.indent(pprint.pformat(default_value), places=4))
         if errors.get(key) is not None:
-            lines.append(u'  Error: %s' % errors[key])
+            lines.append('  Error: %s' % errors[key])
     return '\n'.join(lines)
 
 
 def mask_value_if_secret(key, value):
     if key.endswith('PASSWORD') and value:
-        return u'********'
+        return '********'
     else:
         return value
 
@@ -229,19 +235,19 @@ def did_you_mean(setting, defaults):
     return None
 
 
-def levenshtein(a, b, max=3):
+def levenshtein(a, b):
     """Calculates the Levenshtein distance between a and b."""
     n, m = len(a), len(b)
     if n > m:
         return levenshtein(b, a)
 
-    current = xrange(n+1)
-    for i in xrange(1, m+1):
+    current = xrange(n + 1)
+    for i in xrange(1, m + 1):
         previous, current = current, [i] + [0] * n
-        for j in xrange(1, n+1):
-            add, delete = previous[j] + 1, current[j-1] + 1
-            change = previous[j-1]
-            if a[j-1] != b[i-1]:
+        for j in xrange(1, n + 1):
+            add, delete = previous[j] + 1, current[j - 1] + 1
+            change = previous[j - 1]
+            if a[j - 1] != b[i - 1]:
                 change += 1
             current[j] = min(add, delete, change)
     return current[n]
